@@ -9,13 +9,14 @@ require Exporter;
 # Paths should have no trailing slash.
 #
 # Specify a directory for the application schema (and recompile):
+# (override with $ENV{OSPERL_SCHEMA_DB})
 
-sub SCHEMA_DBDIR() { '/opt/os/joshua' }
+sub SCHEMA_DBDIR() { 'elvis:/opt/os/joshua' }
 
 
 # Specify a directory for temporary databases (posh, osp_copy, etc):
 
-sub TMP_DBDIR() { '/opt/os/tmp' }
+sub TMP_DBDIR() { 'elvis:/opt/os/tmp' }
 
 
 1;
